# -*- coding: utf-8 -*-

class ArabicNumerics(object):
    arabic_number_translation_src = "٠١٢٣٤٥٦٧٨٩٪۰۱۲۳۴۵۶۷۸۹"
    arabic_number_translation_dst = "0123456789%0123456789"
    def __init__(self):
        pass
